
package org.eclipse.birt.core.archive.compound;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import junit.framework.TestCase;

import org.eclipse.birt.core.util.IOUtil;

public class BlockManagerTest extends TestCase
{

	public void testBlockManager( ) throws IOException
	{
		new File( "test.rf" ).delete( );
		RandomAccessFile rf = new RandomAccessFile( "test.rf", "rw" );
		BlockManager manager = new BlockManager( new CacheManagerEventAdapter(
				rf ) );
		manager.setCacheSize( 13 );

		int TEST_COUNT = 1000;
		int BLOCK_COUNT = 100;

		byte[] b = new byte[4];
		for ( int i = 0; i < BLOCK_COUNT; i++ )
		{
			Block block = manager.getBlock( i );
			int v = i;
			IOUtil.integerToBytes( v, b );
			block.write( 0, b, 0, 4 );
		}

		for ( int i = 0; i < TEST_COUNT; i++ )
		{
			int id = (int) ( Math.random( ) * BLOCK_COUNT );
			Block block = manager.getBlock( id );
			block.read( 0, b, 0, 4 );
			int v = IOUtil.bytesToInteger( b );
			assertEquals( id, v );
		}

		manager.flush( );
		rf.close( );

		rf = new RandomAccessFile( "test.rf", "r" );
		for ( int i = 0; i < BLOCK_COUNT; i++ )
		{
			rf.seek( i * Block.BLOCK_SIZE );
			int v = rf.readInt( );
			assertEquals( v, i );
		}
		rf.close( );

		new File( "test.rf" ).delete( );
	}

	class CacheManagerEventAdapter extends BlockManagerEventAdapter
	{

		RandomAccessFile rf;

		CacheManagerEventAdapter( RandomAccessFile rf )
		{
			this.rf = rf;
		}

		public void flush( Block block ) throws IOException
		{
			block.flush( rf );

		}

		public void refresh( Block block ) throws IOException
		{
			block.refresh( rf );
		}

	}
}
