/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.command;

import java.util.List;

import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.DesignEngine;
import org.eclipse.birt.report.model.core.DesignElement;
import org.eclipse.birt.report.model.parser.DesignSchemaConstants;
import org.eclipse.birt.report.model.util.BaseTestCase;

/**
 * Unit test for class <code>TemplateCommand</code>.
 * 
 * <p>
 * <table border="1" cellpadding="2" cellspacing="2" style="border-collapse:
 * collapse" bordercolor="#111111">
 * <th width="20%">Method</th>
 * <th width="40%">Test Case</th>
 * <th width="40%">Expected</th>
 * 
 * <tr>
 * <td>{@link #checkAdd( Object content, int slotID )}</td>
 * <td>Test when add a template parameter, at the same time add a template
 * definition.</td>
 * <td>Both template parameter and template definition are copied to the new
 * design file</td>
 * </tr>
 * 
 * 
 * </table>
 * 
 */

public class TemplateCommandTest extends BaseTestCase
{

	/**
	 * Template element in design file
	 */
	private DesignElement templateItem = null;
	/**
	 * List element in design file
	 */
	private DesignElement designElement = null;

	/**
	 * Input,output, golden file
	 */
	private static final String INPUT_FILE = "TemplateCommandTest.xml";//$NON-NLS-1$
	private static final String OUTPUT_FILE = "TemplateCommandTest_out.xml"; //$NON-NLS-1$
	private static final String GOLEAN_FILE = "TemplateCommandTest_golden.xml"; //$NON-NLS-1$

	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp( ) throws Exception
	{
		super.setUp( );

		getParam( );
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown( ) throws Exception
	{
		super.tearDown( );
		templateItem = null;
		designElement = null;
	}

	/**
	 * Init template element and list element which contains a template element
	 * 
	 * @throws Exception
	 */
	private void getParam( ) throws Exception
	{
		openDesign( INPUT_FILE );

		List list = designHandle.getBody( ).getContents( );
		designElement = (DesignElement) ( (DesignElementHandle) list.get( 0 ) )
				.getElement( ).clone( );
		templateItem = (DesignElement) ( (DesignElementHandle) list.get( 1 ) )
				.getElement( ).clone( );
		designHandle.close( );
	}

	/**
	 * Test when add a template parameter ,at the same time add a template
	 * definition. Test checkAdd( Object content, int slotID ) method
	 * 
	 * @throws Exception
	 */

	public void testCheckAdd( ) throws Exception
	{
		createDesign( );
		design.getVersionManager( ).setVersion(
				DesignSchemaConstants.REPORT_VERSION );

		DesignEngine.getMetaDataDictionary( ).enableElementID( );
		ContentCommand command = new ContentCommand( design, design );
		command.add( designElement, designHandle.BODY_SLOT );
		command.add( templateItem, designHandle.BODY_SLOT );
		saveAs( OUTPUT_FILE );

		compareTextFile( GOLEAN_FILE, OUTPUT_FILE );
	}
}
