/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.tests.box;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.birt.report.model.api.DesignElementHandle;
import org.eclipse.birt.report.model.api.ModuleHandle;
import org.eclipse.birt.report.model.api.extension.ExtendedElementException;
import org.eclipse.birt.report.model.api.extension.IElementCommand;
import org.eclipse.birt.report.model.api.extension.IPropertyDefinition;
import org.eclipse.birt.report.model.api.extension.IReportItem;
import org.eclipse.birt.report.model.api.extension.IReportItemFactory;
import org.eclipse.birt.report.model.api.extension.ReportItem;

/**
 * Implements <code>IReportItem</code> for testing.
 */

public class ReportItemImpl extends ReportItem implements IReportItem
{

	public static final String USAGE_PROP = "usage"; //$NON-NLS-1$

	protected IReportItemFactory cachedDefn = null;
	protected ModuleHandle moduleHandle = null;
	protected DesignElementHandle extItemHandle = null;

	private String usage;

	private boolean refreshNeeded = false;

	/**
	 * Constructs an element.
	 * 
	 * @param extDefn
	 * @param elementHandle
	 */

	public ReportItemImpl( IReportItemFactory extDefn,
			DesignElementHandle elementHandle )
	{
		this.cachedDefn = extDefn;
		assert elementHandle != null;
		this.moduleHandle = elementHandle.getModuleHandle( );
		this.extItemHandle = elementHandle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#serialize(java.lang.String)
	 */
	public ByteArrayOutputStream serialize( String propName )
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#deserialize(java.lang.String,
	 *      java.io.ByteArrayInputStream)
	 */
	public void deserialize( String propName, ByteArrayInputStream data )
			throws ExtendedElementException
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#getProperty(java.lang.String)
	 */
	public Object getProperty( String propName )
	{
		if ( USAGE_PROP.equalsIgnoreCase( propName ) )
		{
			return usage;
		}

		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#checkProperty(java.lang.String,
	 *      java.lang.Object)
	 */
	public void checkProperty( String propName, Object value )
			throws ExtendedElementException
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#setProperty(java.lang.String,
	 *      java.lang.Object)
	 */
	public void setProperty( String propName, Object value )
	{
		moduleHandle.getCommandStack( ).execute(
				getElementCommand( propName, value ) );
	}

	public void doSetProperty( String propName, Object value )
	{
		if ( value == null )
			return;

		if ( USAGE_PROP.equalsIgnoreCase( propName ) )
		{
			usage = value.toString( );
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#validate()
	 */
	public List validate( )
	{
		ExtendedElementException exception = new ExtendedElementException(
				extItemHandle.getElement( ),
				"test.testingbox.plugin", "1", null );//$NON-NLS-1$ //$NON-NLS-2$
		exception.setProperty( ExtendedElementException.LINE_NUMBER, "15" ); //$NON-NLS-1$
		exception.setProperty( ExtendedElementException.LOCALIZED_MESSAGE,
				"local actuate" ); //$NON-NLS-1$
		List list = new ArrayList( );
		list.add( exception );

		return list;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#copy()
	 */
	public IReportItem copy( )
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.model.extension.IExtendedElement#getElementCommand(java.lang.String,
	 *      java.lang.Object)
	 */
	public IElementCommand getElementCommand( String propName, Object value )
	{
		return new ElementCommandImpl( this, propName, value, extItemHandle );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.model.extension.IElement#getPropertyDefinitions()
	 */
	public IPropertyDefinition[] getPropertyDefinitions( )
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.model.extension.IElement#refreshPropertyDefinition()
	 */
	public boolean refreshPropertyDefinition( )
	{
		if ( refreshNeeded )
		{
			refreshNeeded = false;
			return true;
		}

		return false;
	}

	public DesignElementHandle getExtItemHandle( )
	{

		return extItemHandle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.model.api.extension.IReportItem#getMethods()
	 */

	public IPropertyDefinition[] getMethods( )
	{
		return null;
	}
}